/*
Desenvolvido por Francisco Diego Lima Moreira
Fundador do Canal Silícios Lab - https://www.youtube.com/SilíciosLab
*/

#include <DS1307.h>
#include <Wire.h>
#include <EEPROM.h>
#include <LiquidCrystal.h>

#define MEMORIA 120

const int rs = 2, en = 3, d4 = 4, d5 = 5, d6 = 6, d7 = 7;
LiquidCrystal lcd(rs, en, d4, d5, d6, d7);

char tempo[9] = "";
char data[9] = "";

int DataTime[7];

byte SegAnt = 0, SegAtual = 0;

byte LerTeclado(void);
void AjustarHora();


void setup()
{
      Serial.begin(9600);
      DS1307.begin();
      //Config pins to control columns of the matrix keyboard
      for(int tecla = 8; tecla < 12; tecla++)
      {
        pinMode(tecla, OUTPUT);  
      }

      //Config pins to read lines of the matrix keyboard
      for(int tecla = 12; tecla < 16; tecla++)
      {
        pinMode(tecla, INPUT);  
      }

      for(int tecla = 8; tecla < 12; tecla++)
      {
        digitalWrite(tecla, LOW);  
      }

      lcd.begin(16,2);
}

void loop()
{
    if(EEPROM.read(MEMORIA) != 73)
    {
      AjustarHora();  
      EEPROM.write(MEMORIA, 73);
    }
  
    DS1307.getDate(DataTime);    
    SegAtual = DataTime[6];

    if(LerTeclado() == 10)
    {
      AjustarHora();
    }
    
    if(abs(SegAtual - SegAnt) >= 1)
    {
    sprintf(tempo, "%02d:%02d:%02d", DataTime[4], DataTime[5], DataTime[6]);
    sprintf(data, "%02d/%02d/%02d", DataTime[2], DataTime[1], DataTime[0]);

    Serial.println(tempo);
    Serial.println(data);
    
    lcd.setCursor(4,1);
    lcd.print(tempo);
    lcd.setCursor(4,0);
    lcd.print(data);
    SegAnt = SegAtual;
    }
}

byte LerTeclado(void)
{   
  byte teclado[4][4] = {
  {1,2,3,10},
  {4,5,6,11},
  {7,8,9,12},
  {14,0,15,13}};
    
    byte coluna[4] = {8,9,10,11};
    byte linha[4]  = {12,13,14,15};
    bool tecla;
    
    byte digito_tecla = 16;

    for(byte y = 0; y <= 3; y++)
    {
      digitalWrite(coluna[y], LOW);
    }
    
    for(byte y = 0; y <= 3; y++)
    {
      
      digitalWrite(coluna[y], HIGH);

        for(byte x = 0; x <= 3; x++)
        {
            tecla = digitalRead(linha[x]);  
            
            if(tecla == 1)
            {
              digito_tecla = teclado[x][y];
              return digito_tecla;
            }             
        }

     digitalWrite(coluna[y], LOW);
    }

    return digito_tecla;
}

void AjustarHora()
{
    int times[6];
    int  DadosTempo[7];
    bool controle = 0;
    bool EstadoSobe = 0, EstadoDesce = 0;
    
    byte cont = 0;
    byte number = 0;
    int digitos[6];
    byte PosCursor = 0;
    
    bool Validate = 0;
    char tempo[7] = "";
    char data[15] = "";
    char DiaSemana[7][15] = {"-Domingo", "-Segunda-Feira", "-Terca-Feira", "-Quarta-Feira", "-Quinta-Feira", "-Sexta-Feira", "-Sabado"};

    DS1307.getDate(DadosTempo);
    
    sprintf(tempo, "%02d:%02dh", DadosTempo[4], DadosTempo[5]);
    sprintf(data, "%02d/%02d/%02d", DadosTempo[2], DadosTempo[1], DadosTempo[0]);
    
          times[4] = DadosTempo[4]; //Armazena a hora em times[4]
          times[5] = DadosTempo[5]; //Armazena os minutos em times[5]
          times[3] = DadosTempo[3]; //Armazena o dia da semana em times[3]
          times[2] = DadosTempo[2]; //Armazena a Data em times[2]
          
          times[1] = DadosTempo[1]; //Armazena o mes em times[1]
          times[0] = DadosTempo[0]; //Armazena o ano em times[0]
          times[6] = DadosTempo[6]; //Armazena os segundos em times[6]
                
          digitos[0] = times[4]/10; //Armazena a Dezena das Horas
          digitos[1] = times[4]%10; //Armazena a Unidade das Horas
          digitos[2] = times[5]/10; //Armazena a Dezena dos Minutos
          digitos[3] = times[5]%10; //Armazena a Unidade dos Minutos
    
          do
          {
            
              lcd.setCursor(0,0);
              lcd.print("                ");
              lcd.setCursor(2,0);  
              lcd.print("Ajustar hora:");
              lcd.setCursor(0,1);
              lcd.print("                ");
              lcd.setCursor(5,1);
              lcd.print(tempo);
              PosCursor = 5;
              Validate = 0;
              
            do
            {
              number = LerTeclado();
              
              delay(100);
              
              if( (number >= 0 && number <= 9) && (controle == 0) && (cont < 4) )
              {Serial.println(number);
                digitos[cont] = number;  
                cont++;
                controle = 1;
                
                lcd.setCursor(PosCursor,1);
                
                lcd.print(number);                
                PosCursor++;
                
                if(cont == 2 || cont == 4)
                {
                  PosCursor = PosCursor + 1;
                }
              }
              
              if(number == 16 && controle == 1)
              {
                controle = 0;
              }
              
              if(number == 12)
              { 
                  for(cont = 0; cont < 4; cont++)
                  {
                    digitos[cont] = 0;
                  }  
                
                lcd.setCursor(0,1);
                lcd.print("                ");
                lcd.setCursor(5,1);
                lcd.print("00:00h");
                PosCursor = 5;
                cont = 0;

                for(byte i = 0; i < 4; i++)
                {
                  digitos[i] = 0;  
                }
              }
              
            }while(number != 13);  

            times[4] = (digitos[0]*10) + digitos[1];
            times[5] = (digitos[2]*10) + digitos[3];

            Serial.println(times[4]);
            Serial.println(times[5]);
            
            if((times[4] < 0 || times[4] > 23) || (times[5] < 0 || times[5] > 59))
            {
              lcd.clear();
              lcd.setCursor(6,0);
              lcd.print("Hora");
              lcd.setCursor(4,1);  
              lcd.print("Invalida");
              delay(1500);
              Validate = 1;

              lcd.clear();
              
              lcd.setCursor(0,0);
              lcd.print("                ");
              lcd.setCursor(1,0);  
              lcd.print("Ajustar horas:");
              lcd.setCursor(0,1);
              lcd.print("                ");
              lcd.setCursor(5,1);
              lcd.print("00:00h");
              
              sprintf(tempo, "%02d:%02dh", 0, 0);
              
              PosCursor = 5;
              
              cont = 0;

                for(byte i = 0; i < 4; i++)
                {
                  digitos[i] = 0;  
                }

                for(byte i = 0; i < 6; i++)
                {
                  times[i] = 0;  
                }
            }
          
          }while(Validate == 1);

              do
              {
                number = LerTeclado();
                delay(200);
              }while(number != 16);


          
/*----------------------------------------Configuracao da Data-----------------------------------------------*/
            PosCursor = 4;
            
            do
            {
              //Transformacao dos digitos da data para dezenas e unidades
              digitos[0] = times[2]/10; //Armazena a Dezena da Data
              digitos[1] = times[2]%10; //Armazena a Unidade da Data
              digitos[2] = times[1]/10; //Armazena a Dezena do Mes
              digitos[3] = times[1]%10; //Armazena a Unidade do Mes
              digitos[4] = times[0]/10; //Armazena a Dezena do Ano
              digitos[5] = times[0]%10; //Armazena a Unidade do Ano
              Validate = 0;
              
              sprintf(data, "%02d/%02d/%02d", times[2], times[1], times[0]);
                lcd.clear();
                lcd.setCursor(0,0);
                lcd.print("                ");
                lcd.setCursor(2,0);  
                lcd.print("Ajustar Data:");           
                lcd.setCursor(0,1);
                lcd.print("                ");
                lcd.setCursor(4,1);
                lcd.print(data);
                PosCursor = 4;
                cont = 0;
            
            do
            {
              
              number = LerTeclado();
              delay(100);
              
              if( (number >= 0 && number <= 9) && (controle == 0) && (cont < 6) )
              {
                digitos[cont] = number;
                  
                cont++;
                controle = 1;              
                
                lcd.setCursor(PosCursor,1);
                
                lcd.print(number);                
                PosCursor++;
                
                if(cont == 2 || cont == 4)
                {
                  PosCursor = PosCursor + 1;
                }
              }
              
              if(number == 16 && controle == 1)
              {
                controle = 0;
              }
              
              if(number == 12)
              { 
                  for(cont = 0; cont < 6; cont++)
                  {
                    digitos[cont] = 0;
                  }  
                
                lcd.setCursor(0,1);
                lcd.print("                ");
                lcd.setCursor(4,1);
                lcd.print("00/00/00");
                PosCursor = 4;
                cont = 0;
              }
              
            }while(number != 13);
            
            times[2] = (digitos[0]*10) + digitos[1]; //Transformando os numeros lidos para data do mes em dois dígitos
            times[1] = (digitos[2]*10) + digitos[3]; //Transformando os numeros lidos para mes em dois dígitos
            times[0] = (digitos[4]*10) + digitos[5]; //Transformando os numeros lidos para ano em dois dígitos
                
            if((times[2] <= 0 || times[2] > 31) || (times[1] <= 0 || times[1] > 12) || times[0] <= 0 || (times[2] > 28 && times[1] == 2))
            {
                lcd.clear();
                lcd.setCursor(6,0);
                lcd.print("Data");
                lcd.setCursor(4,1);  
                lcd.print("Invalida");
                delay(1500);
                Validate = 1;
  
                lcd.clear();
              
                lcd.setCursor(0,0);
                lcd.print("                ");
                lcd.setCursor(2,0);  
                lcd.print("Ajustar Data:");           
                lcd.setCursor(0,1);
                lcd.print("                ");
                lcd.setCursor(3,1);
                lcd.print("00/00/00");
                PosCursor = 5;
                cont = 0;

                for(byte i = 0; i < 6; i++)
                {
                  digitos[i] = 0;  
                }

                for(byte i = 0; i < 3; i++)
                {
                  times[i] = 0;  
                }
                
            }
          
          }while(Validate == 1);

          do
          {
            number = LerTeclado();
            delay(200);
          }while(number != 16);
          
/*----------------------------------------------Configuracao dos Dias da Semana------------------------------------------------------*/
          lcd.clear();
          signed ContDias = times[3];
          
          lcd.setCursor(1,0);
          lcd.print("Ajuste o Dia:");
          lcd.setCursor(0,1);
          lcd.print(DiaSemana[ContDias]); 

          do
          {
            number = LerTeclado();
            delay(100);

            if(number == 10 && EstadoSobe == 0)
            {
              ContDias++;    

              if(ContDias > 6)
              {
                ContDias = 0;  
              }
              
              lcd.setCursor(0,1);
              lcd.print("                ");
              lcd.setCursor(1,1);
              lcd.print(DiaSemana[ContDias]);   
              EstadoSobe = 1;       
            }

            if(number == 16 && EstadoSobe == 1)
            {
              EstadoSobe = 0;
            }

            if(number == 11 && EstadoDesce == 0)
            {
              ContDias--;

              if(ContDias < 0)
              {
                ContDias = 6;  
              }
              
              lcd.setCursor(0,1);
              lcd.print("                ");
              lcd.setCursor(1,1);
              lcd.print(DiaSemana[ContDias]);  
              EstadoDesce = 1;
            }

            if(number == 16 && EstadoDesce == 1)
            {
              EstadoDesce = 0;
            }
            
          }while(number != 13);
          
            times[3] = ContDias;

            lcd.clear();
            
           DS1307.setDate(times[0],times[1],times[2],times[3],times[4],times[5],00);//ano,mes,dia,semana,horas,minutos,segundo
}
